/*
mu.c

This program read metadata from archive HDF file and compare 
it with inventory metadata. Then update and write metadata 
back to the HDF file.

Zhangshi Yin
 
2002.1.2.
*/


#include "mfhdf.h"
#include <stdio.h>
#include <stdlib.h>
#include "odldef.h"


/* Write metadataFile to HDFName. */ 
int writeMetadata(char *HDFName, char *attr, char *groupName, FILE *updateMetadataFile);

/* Write metadataFile to hdfFile. */ 
intn writeAttr(int32 hdfFile, char *attr, char *groupName, FILE *updateMetadataFile);

/* Read metadata from HDFName and write temp file. */
int readMetadata(char *HDFName, char *attr);

/* Read metadata from hdfFile and write temp file. */
intn readAttr(int32 hdfFile, char *attr);

/* Convert low case to upcase. */ 
void LtoU (char *inLine, char *outLine);

/* Convert space line to no space line. */
void noSpaceLine(char *inLine, char *outLine);

/* Check attribute value. */
intn checkValue(FILE *metadataFile, FILE *tempInventoryFile, char *attribute);

/* Delete temporary file. */
void deleteTempFile();

/* Get a value of ODL attribute. */
intn getValue(FILE *metadataFile,  char *attributeName, char *valueName, char *retrunValue);


main(int argc, char *argv[])
{
  intn i, status, hFile=0, mFile=0, oFile=0, verbose=0, inventory=0, collection=0;
  char *HDFName, *metadataName, *outputName, yn[10];
  char oLine[1024], aLine[1024], tempLine[1024];
  char inLine[1024], outLine[1024], cmdstr[1024], attribute[1024];
  char HDFValue[256], METValue[256], attributeName[256], valueName[256];
  char returnValue[256];
  FILE *metadataFile, *tempArchiveFile, *tempInventoryFile;
  AGGREGATE root, roota, aAggregate, bAggregate, cAggregate, dAggregate, eAggregate;
  PARAMETER aParameter, bParameter;
  VALUE aValue;

  /* 
  Check for arguments. 
  */
  if (argc==1) 
  {
    printf("\n");
    printf("Usage: \n");
    printf("mu -h inputHDFFile -m inputMetadataFile -o outputHDFFile -v\n");
    printf("\n");
    printf("   inputHDFFile--HDFFile to be updated.\n");
    printf("   inputMetadataFile--Metadata file for inputHDFFile.\n");
    printf("   outputHDFFile--Update to a new file.\n");
    printf("   If no -o flag, update inputHDFFile.\n");
    printf("   If -v flag, display verbose message on screen.\n");
    printf("\n");
    exit(1);
  }

  /*
  Get file names from arguments.
  */
  for (i=0; i<argc; i++)
  {
    if (strcmp(argv[i], "-h")==0)
    {
      HDFName=argv[i+1];
      printf("\n");
      printf("Input HDF-EOS file is %s.\n", argv[i+1]);
      hFile=1;
    }
    if (strcmp(argv[i], "-m")==0)
    {
      metadataName=argv[i+1];
      printf("Input metadata file is %s.\n", argv[i+1]);
      mFile=1;
    }
    if (strcmp(argv[i], "-o")==0)
    {
      outputName=argv[i+1];
      oFile=1;
    }
    if (strcmp(argv[i], "-v")==0)
    {
      verbose=1;
    }
  }

  if (hFile==0)
  {
    fprintf(stderr, "No HDF file for input.\n");
    exit(1);
  }

  if (mFile==0)
  {
    fprintf(stderr, "No metadata file for input.\n");
    exit(1);
  }

  /*
  Copy HDF file to  a new file to update.
  */
  if (verbose)
  {
    printf("Check output files.\n");
  }
  if (oFile==0)
  {
    printf("Warning: This program will update input HDF file.\n");
    printf("         If you want keep original file and update to a new file,\n");
    printf("         please use -o option.\n");
    yn:
    printf("         Do you want to update original file?\n");
    printf("         (y/n):");
    scanf("%s", yn);
    if(strcmp(yn, "y")==0)
    {
    }
    else if(strcmp(yn, "n")==0)
    {
      exit(0);
    }
    else
    {
      goto yn;
    }
  }
  else
  {
    if (fopen(HDFName, "r")==NULL)
    {
      fprintf(stderr, "HDF-EOS file doesn't exist.\n");
      exit(1);
    }
    if (verbose)
    {
      printf("Copy HDF file to output file.\n");
    }
    strcpy(cmdstr, "cp ");
    strcat(cmdstr, HDFName);
    strcat(cmdstr, " ");
    strcat(cmdstr, outputName);
    system(cmdstr);  
    strcpy(HDFName, outputName);
  }

  /* 
  Read metadata from hdf file. Write to tempInventory and tempArchive files. 
  */
  printf("Open HDF-EOS file and read metadata from HDF-EOS file.\n");
  status = readMetadata(HDFName, "CoreMetadata");
  status = readMetadata(HDFName, "coremetadata");
  status = readMetadata(HDFName, "ArchiveMetadata");
  status = readMetadata(HDFName, "archivemetadata");
  /*
  status = readMetadata(HDFName, "InventoryMetadata");
  status = readMetadata(HDFName, "inventorymetadata");
  status = readMetadata(HDFName, "Collection");
  status = readMetadata(HDFName, "collection");
  status = readMetadata(HDFName, "CollectionMetadata");
  status = readMetadata(HDFName, "collectionmetadata");
  status = readMetadata(HDFName, "StructMetadata");
  status = readMetadata(HDFName, "structmetadata");
  status = readMetadata(HDFName, "ProductMetadata");
  status = readMetadata(HDFName, "productmetadata");
  */
 
  /* Check temp inventory file. */ 
  if (verbose)
  {
    printf("Check inventory and archive files.\n");
  }
  tempInventoryFile=fopen("tempInventoryFile", "r");
  if (tempInventoryFile==NULL)
  {
    fprintf(stderr, "No inventory metadata in HDF file or\n");
    fprintf(stderr, "inventory metadata cannot be read.\n");
  }
  else
  {
    printf("There is inventory metadata in HDF file.\n");
  }

  /* Check temp archive file. */
  tempArchiveFile=fopen("tempArchiveFile", "r");
  if (tempArchiveFile==NULL)
  {
    fprintf(stderr, "There is no archive metadata in HDF file or\n");
    fprintf(stderr, "archive metadata cannot be read.\n");
  }
  else
  {
    printf("There is archive metadata in HDF file.\n");
  }

  /*
  Check syntax of tempInventoryFile.
  */
  printf("Check syntax of HDF-EOS metadata.\n");
  if (tempInventoryFile!=NULL)
  {
    printf("Check syntax of inventory metadata.\n");
    fseek(tempInventoryFile, 0, 0);
    root=NewAggregate(NULL, KA_GROUP, "test", NULL);
    status=ReadLabel(tempInventoryFile, root);
    if (status!=1)
    {
      deleteTempFile();
      fprintf(stderr, "There are syntax errors in inventory metadata.\n");
      exit(1);
    }
    RemoveAggregate(root);
  }

  /*
  Check syntax of tempArchiveFile.
  */
  if (tempArchiveFile!=NULL)
  {
    printf("Check syntax of archive metadata.\n");
    fseek(tempArchiveFile, 0, 0);
    root=NewAggregate(NULL, KA_GROUP, "test", NULL);
    status=ReadLabel(tempArchiveFile, root);
    if (status!=1)
    {
      deleteTempFile();
      fprintf(stderr, "There are syntax errors in archive metadata.\n");
      exit(1);
    }
    RemoveAggregate(root);
  }

  /*
  Open metadata file. 
  */ 
  printf("Open metadata file.\n");
  metadataFile = fopen(metadataName, "r");
  if (metadataFile==NULL)
  {
    deleteTempFile();
    fprintf(stderr, "Metadata file %s cannot be opened.\n", metadataName);
    exit(1);
  }

  /*
  Check syntax of metadata file.
  */
  printf("Check syntax of metadata file.\n");
  fseek(metadataFile, 0, 0);
  root=NewAggregate(NULL, KA_GROUP, "test", NULL);

  /*
  Read odl, no error return 1 and error return other.
  */
  status=ReadLabel(metadataFile, root);
  if (status!=1)
  {
    deleteTempFile();
    fprintf(stderr, "There are syntax errors in metadata file.\n");
    exit(1);
  }
  /*
  roota=NewAggregate(NULL, KA_GROUP, "test", NULL);
  bAggregate=FindGroup(root, "INVENTORYMETADATA");
  cAggregate=PasteAggregate(roota, bAggregate);
  PrintLabel(cAggregate);
  PrintLabel(root->first_child);
  PrintLabel(root->first_child->right_sibling);
  PrintLabel(root->first_child->right_sibling->right_sibling);
  */
  if (verbose)
  {
    printf("Check collection and inventory metadata.\n");
  }
  if (FindGroup(root, "COLLECTIONMETADATA")!=NULL)
  {
    collection=1;
    printf("There is collection metadata in metadata file.\n");
  }
  else
  {
    printf("There is no collection metadata in metadata file.\n");
  } 
  if (FindGroup(root, "INVENTORYMETADATA")!=NULL)
  {
    inventory=1;
    printf("There is inventory metadata in metadata file.\n");
  }
  else
  {
    printf("There is no inventory metadata in metadata file.\n");
  }

  RemoveAggregate(root);

  /*
  Check inventory metadata values:
  If values are equal, continue. If not equal, display message and exit. 
  */
  printf("Check inventory metadata values from HDF and metadata files.\n"); 
  /* Call comparison function. */
  if (tempInventoryFile!=NULL && metadataFile!=NULL)
  {
    /* Rewind metadataFile */
    fseek(tempInventoryFile, 0, 0);
    fseek(metadataFile, 0, 0);

    if (verbose)
    {
      printf("Check shortName value.\n");
    }
    strcpy(attributeName, "SHORTNAME");
    strcpy(valueName, "VALUE");
    getValue(tempInventoryFile, attributeName, valueName, HDFValue);
    getValue(metadataFile, attributeName, valueName, METValue);
    if (strcmp(HDFValue, METValue)!=0)
    {  
      printf("Attribute %s values: %s and %s are not equal.\n", attributeName, HDFValue, METValue);
      deleteTempFile();
      exit(1);
    }
    if (verbose)
    {
      printf("Check productionDateTime value.\n");
    }
    strcpy(attributeName, "PRODUCTIONDATETIME");
    strcpy(valueName, "VALUE");
    getValue(tempInventoryFile, attributeName, valueName, HDFValue);
    getValue(metadataFile, attributeName, valueName, METValue);
    if (strcmp(HDFValue, METValue)!=0)
    {  
      printf("Attribute %s values: %s and %s are not equal.\n", attributeName, HDFValue, METValue);
      deleteTempFile();
      exit(1);
    }
    if (verbose)
    {
      printf("Check versionID value.\n");
    }
    strcpy(attributeName, "VERSIONID");
    strcpy(valueName, "VALUE");
    getValue(tempInventoryFile, attributeName, valueName, HDFValue);
    getValue(metadataFile, attributeName, valueName, METValue);
    if (strcmp(HDFValue, METValue)!=0)
    {  
      printf("Attribute %s values: %s and %s are not equal.\n", attributeName, HDFValue, METValue);
      deleteTempFile();
      exit(1);
    }
  }

  /*
  If there are updated metadata, write back to HDF file.
  */
  printf("Updating metadata in HDF file.\n");
  if (collection==1)
  {
    printf("Updating collection metadata.\n"); 
    fseek(metadataFile, 0, 0);
    writeMetadata(HDFName, "CollectionMetadata", "COLLECTIONMETADATA", metadataFile);
  }
  else
  {
    printf("No collection metadata in metadata file and metadata is not updated.\n"); 
  }
  if (inventory==1)
  {
    printf("Updating inventory metadata.\n"); 
    fseek(metadataFile, 0, 0);
    writeMetadata(HDFName, "CoreMetadata", "INVENTORYMETADATA", metadataFile);
  }
  else
  {
    printf("No inventory metadata in metadata file and metatdata is not updated.\n"); 
  }
  if (tempArchiveFile!=NULL)
  {
    printf("Updating archive metadata.\n"); 
    fseek(tempArchiveFile, 0, 0);
    writeMetadata(HDFName, "ArchiveMetadata", "ARCHIVEMETADATA", tempArchiveFile);
  }
  else
  {
    printf("No archive metadata in HDF-EOS file and metadata is not updated.\n"); 
  }

  /* 
  If verbose, write updated metadata to screen.
  */
  if (verbose)
  {
    printf("Display updated metadata:\n");
    fseek(metadataFile, 0, 0);
    root=NewAggregate(NULL, KA_GROUP, "test", NULL);
    status=ReadLabel(metadataFile, root);
    aAggregate=FindGroup(root, "COLLECTIONMETADATA");
    if (aAggregate!=NULL)
    {
      PrintLabel(aAggregate);
    }
    aAggregate=FindGroup(root, "INVENTORYMETADATA");
    if (aAggregate!=NULL)
    {
      PrintLabel(aAggregate);
    }
    RemoveAggregate(root);

    if (tempArchiveFile!=NULL)
    {
      fseek(tempArchiveFile, 0, 0);
      while (fgets(aLine, 1024, tempArchiveFile))
      {
        printf("%s", aLine);
      }
    }
  }

  /* Close metadata and temporary file. */
  if (verbose)
  {
    printf("Close files.\n");
  }
  fclose(metadataFile);
  if (tempArchiveFile!=NULL)
  {
    fclose(tempArchiveFile);
  }
  if (tempInventoryFile!=NULL)
  {
    fclose(tempInventoryFile);
  }

  /* Remove temporary files. */
  if (verbose)
  {
    printf("Delete temporary files.\n");
  }
  deleteTempFile();

  printf("Updating is successfully compeleted.\n");
  printf("Updated file is %s.\n", HDFName);
  printf("\n");
}


/* Read metadata from HDFName and write temp file. */
intn readMetadata(char *HDFName, char *attr)
{
  intn i;
  int32 hdfFile;
  char attrname[100];

  hdfFile = SDstart(HDFName, DFACC_READ);
  if (hdfFile == -1) 
  {
    fprintf(stderr, "Unable to open HDF file %s \n", HDFName);
    deleteTempFile();
    exit(1);
  }

  /* First, try attr name alone */
  readAttr(hdfFile, attr);

  /* Now, try concatenating sequence numbers */
  for (i = 0; i <= 9; i++) 
  {
    sprintf(attrname, "%s.%d", attr, i);
    readAttr(hdfFile, attrname);
  }

  SDend(hdfFile);
  return (0);
}


/* Read metadata from hdfFile and write temp file. */
intn readAttr(int32 hdfFile, char *attr)
{
  intn status = 0;
  int32 attr_index, data_type, totalSize;
  char tempString[50], *astring, *metadata, attr_name[MAX_NC_NAME];
  FILE *tempFile;

  /* Look for attribute */
  attr_index = SDfindattr(hdfFile, attr);
  if (attr_index == -1) 
  {
    /* fprintf(stderr, "Unable to find attribute %s.\n", attr); */
    return (1);
  }
 
  /* Get size of attribute  */
  status = SDattrinfo(hdfFile, attr_index, attr_name, &data_type, &totalSize);
  if (status == -1) 
  {
    fprintf(stderr, "SDattrinfo failed for existing attribute %s.\n", attr);
    return (1);
  }
 
  /* Allocate memory for attribute contents */
  metadata = (char *) malloc(totalSize);
  if (metadata == NULL) 
  {
    fprintf(stderr, "Unable to allocate %d bytes for %s.\n", totalSize, attr);
    deleteTempFile();
    exit(1);
  }
 
  /* Read attribute */
  status = SDreadattr(hdfFile, attr_index, metadata);
  if (status == -1) 
  {
    fprintf(stderr, "Unable to read attribute %s.\n", attr);
    free(metadata);
    return (1);
  }
  strcpy(tempString, attr);
  astring=strtok(tempString, ".");
  /* Print out the metadata to a new file. */
  if (strcmp(astring, "ArchiveMetadata")==0 || strcmp(astring, "archivemetadata")==0)
  {
    tempFile=fopen("tempArchiveFile", "w");
    if (tempFile==NULL)
    {
      printf("Cannot write file.\n");
      deleteTempFile();
      exit(1);
    }

    fprintf(tempFile, "%s", metadata);
    fclose(tempFile);
  }
  if (strcmp(astring, "CoreMetadata")==0 || strcmp(astring, "coremetadata")==0)
  {
    tempFile=fopen("tempInventoryFile", "w");
    if (tempFile==NULL)
    {
      printf("Cannot write file.\n");
      deleteTempFile();
      exit(1);
    }

    fprintf(tempFile, "%s", metadata);
    fclose(tempFile);
  }

  /* Clean up and return success */
  free(metadata);
  return (0);
}


/* Write metadataFile to HDFName. */ 
intn writeMetadata(char *HDFName, char *attr, char *groupName, FILE *metadataFile)
{
  int32 hdfFile;

  hdfFile = SDstart(HDFName, DFACC_WRITE);
  if (hdfFile == -1) 
  {
    deleteTempFile();
    fprintf(stderr, "Unable to open file %s \n", HDFName);
    exit(1);
  }

  writeAttr(hdfFile, attr, groupName, metadataFile);

  SDend(hdfFile);

  return (0);
}


/* Write metadataFile to hdfFile. */ 
intn writeAttr(int32 hdfFile, char *attr, char *groupName, FILE *metadataFile)
{
  intn i=0, strsize, status = 0;
  int32 attr_index, data_type, totalSize=0;
  char *metabuffer, *metapoint, attr_name[MAX_NC_NAME];
  char oLine[1024], aLine[1024], tempLine[1024], outLine[1024];
  char attrname[100], startGroup[100], endGroup[100];

  data_type=4;
  strcpy(startGroup, "GROUP=");
  strcat(startGroup, groupName);
  strcpy(endGroup, "END_GROUP=");
  strcat(endGroup, groupName);

  /*
  Maximum length of a vdata record: 65535  <- from the User's Guide
  Maximum order size:  65535    <-The rest of these from ./hdf/src/hlimits.h
  MAX_FIELD_SIZE: 65535
  Max name length: 65
  Max NO. of fields: 256
  MAX Field Name length: 128
  */

  metabuffer=(char *)malloc(65535);
  metapoint=metabuffer;

  fseek(metadataFile, 0, 0);
  
  while (fgets(oLine, 1024, metadataFile))
  {
    LtoU(oLine, aLine);
    strcpy(tempLine, aLine);
    noSpaceLine(tempLine, outLine);
    if (strstr(outLine, startGroup)!=NULL)
    {
      strsize=strlen(oLine);
      totalSize+=strsize;
      memcpy(metapoint, oLine, strsize);
      metapoint+=strsize;
      while (fgets(oLine, 1024, metadataFile))
      {
        strsize=strlen(oLine);
        totalSize+=strsize;
        //Need to add "END\n", so 65535-4=65531
        if (totalSize>65531)
        {
          totalSize-=strsize;
          sprintf(attrname, "%s.%d", attr, i);
          SDsetattr(hdfFile, attrname, data_type, totalSize, metabuffer);
          free(metabuffer);
          i++;
          totalSize=strsize;
          metabuffer=(char *)malloc(65535);
          metapoint=metabuffer;
        }
        memcpy(metapoint, oLine, strsize);
        metapoint+=strsize;
        LtoU(oLine, aLine);
        strcpy(tempLine, aLine);
        noSpaceLine(tempLine, outLine);
        if (strstr(outLine, endGroup)!=NULL) break;
      }
      memcpy(metapoint, "END\n", 4);
      metapoint+=4;
      sprintf(attrname, "%s.%d", attr, i);
      SDsetattr(hdfFile, attrname, data_type, totalSize, metabuffer);
      free(metabuffer);
    }
  }

  return (0);
}


/* Convert low case to upcase. */ 
void LtoU(char *inLine, char *outLine)
{
  int i;
  char astring, bstring;

  for (i=0; i<strlen(inLine); i++)
  {
    astring=inLine[i];
    bstring=toupper(astring);
    outLine[i]=bstring;
  }
  outLine[strlen(inLine)]='\0';
}

/* Change line with space to no space line. */
void noSpaceLine(char *inLine, char *outLine)
{
  char *astring, *bstring;

  //Get no space line.
  strcpy(outLine, "");
  astring=strtok(inLine, "\n");
  bstring=strtok(astring, "	");
  astring=strtok(bstring, " ");
  while(astring)
  {
    strcat(outLine, astring);
    astring=strtok(NULL, " ");
  }
}

/* Delete temporary file. */
void deleteTempFile()
{
  if (fopen("tempArchiveFile", "r")!=NULL)
  {
    system("rm tempArchiveFile");
  }
  if (fopen("tempInventoryFile", "r")!=NULL)
  {
    system("rm tempInventoryFile");
  }
}

/* Check attribute value. */
intn checkValue(FILE *metadataFile, FILE *tempInventoryFile, char *attribute)
{
  int  lineC=1024;
  char oLine[1024], aLine[1024], tempLine[1024], outLine[1024];
  char *restString, oneLine[1024], *astring;
  char object[512], endObject[512], mValue[1024], hValue[1024];
  

  strcpy(mValue, "");
  strcpy(hValue, "");

  strcpy(object, "");
  strcat(object, "OBJECT=");    
  strcat(object, attribute);    

  strcpy(endObject, "");
  strcat(endObject, "END_OBJECT=");    
  strcat(endObject, attribute);    

  fseek(metadataFile, 0, 0);
  fseek(tempInventoryFile, 0, 0);

  while (fgets(oLine, 1024, metadataFile))
  {
    LtoU(oLine, aLine);
    strcpy(tempLine, aLine);
    noSpaceLine(tempLine, outLine);

    if (strstr(outLine, object)!=NULL)
    {
      while (fgets(oLine, 1024, metadataFile))
      {
        LtoU(oLine, aLine);
        strcpy(tempLine, aLine);
        noSpaceLine(tempLine, outLine);

        /* Check ( { " */  
        if (strstr(outLine, "{")!=NULL)
        {
          strcat(oneLine, outLine);
          while (strstr(outLine, "}")==NULL)
          {
            fgets(oLine, 1024, metadataFile);
            LtoU(oLine, aLine);
            strcpy(tempLine, aLine);
            noSpaceLine(tempLine, outLine);
            strcat(oneLine, outLine);
          }
          strcpy(outLine, oneLine); 
        }
        else if (strstr(outLine, "(")!=NULL)
        {
          strcat(oneLine, outLine);
          while (strstr(outLine, ")")==NULL)
          {
            fgets(oLine, 1024, metadataFile);
            LtoU(oLine, aLine);
            strcpy(tempLine, aLine);
            noSpaceLine(tempLine, outLine);
            strcat(oneLine, outLine);
          }
          strcpy(outLine, oneLine); 
        }
        else if (strstr(outLine, "\"")!=NULL)
        {
          strcpy(oneLine, "");
          strcat(oneLine, outLine);
          strcpy(tempLine, outLine);
          restString=strstr(tempLine, "\"");
          restString[0]='a';
          if (strstr(restString, "\"")==NULL)
          {
            fgets(oLine, 1024, metadataFile);
            LtoU(oLine, aLine);
            strcpy(tempLine, aLine);
            noSpaceLine(tempLine, outLine);
            strcat(oneLine, outLine);
            while (strstr(outLine, "\"")==NULL)
            {
              fgets(oLine, 1024, metadataFile);
              LtoU(oLine, aLine);
              strcpy(tempLine, aLine);
              noSpaceLine(tempLine, outLine);
              strcat(oneLine, outLine);
            }
          }
          strcpy(outLine, oneLine); 
        }

        if (strstr(outLine, "VALUE=")!=NULL)
        {
          strcpy(mValue, "");
          strcpy(mValue, outLine);
        }
        if (strstr(outLine, endObject)!=NULL) break;
      }
    }
  }
  while (fgets(oLine, 1024, tempInventoryFile))
  {
    LtoU(oLine, aLine);
    strcpy(tempLine, aLine);
    noSpaceLine(tempLine, outLine);

    if (strstr(outLine, object)!=NULL)
    {
      while (fgets(oLine, 1024, tempInventoryFile))
      {
        LtoU(oLine, aLine);
        strcpy(tempLine, aLine);
        noSpaceLine(tempLine, outLine);

        /* Check ( { " */  
        if (strstr(outLine, "(")!=NULL)
        {
          strcat(oneLine, outLine);
          while (strstr(outLine, ")")==NULL)
          {
            fgets(oLine, 1024, metadataFile);
            LtoU(oLine, aLine);
            strcpy(tempLine, aLine);
            noSpaceLine(tempLine, outLine);
            strcat(oneLine, outLine);
          }
          strcpy(outLine, oneLine); 
        }
        if (strstr(outLine, "{")!=NULL)
        {
          strcat(oneLine, outLine);
          while (strstr(outLine, "}")==NULL)
          {
            fgets(oLine, 1024, metadataFile);
            LtoU(oLine, aLine);
            strcpy(tempLine, aLine);
            noSpaceLine(tempLine, outLine);
            strcat(oneLine, outLine);
          }
          strcpy(outLine, oneLine); 
        }
        if (strstr(outLine, "\"")!=NULL)
        {
          strcpy(oneLine, "");
          strcat(oneLine, outLine);
          strcpy(tempLine, outLine);
          restString=strstr(tempLine, "\"");
          restString[0]='a';
          if (strstr(restString, "\"")==NULL)
          {
            fgets(oLine, 1024, metadataFile);
            LtoU(oLine, aLine);
            strcpy(tempLine, aLine);
            noSpaceLine(tempLine, outLine);
            strcat(oneLine, outLine);
            while (strstr(outLine, "\"")==NULL)
            {
              fgets(oLine, 1024, metadataFile);
              LtoU(oLine, aLine);
              strcpy(tempLine, aLine);
              noSpaceLine(tempLine, outLine);
              strcat(oneLine, outLine);
            }
          }
          strcpy(outLine, oneLine); 
        }

        if (strstr(outLine, "VALUE=")!=NULL)
        {
          strcpy(hValue, "");
          strcpy(hValue, outLine);
        }
        if (strstr(outLine, endObject)!=NULL) break;
      }
    }
  }

  if (strcmp(mValue, hValue)!=0)
  {  
    deleteTempFile();
    printf("Attribute %s values: %s and %s are not equal.\n", attribute, mValue, hValue);
    exit(1);
  }
}

/* Get value of ODL attribute. */
intn getValue(FILE *metadataFile,  char *attributeName, char *valueName, char *returnValue)
{
  AGGREGATE root, roota, aAggregate, bAggregate, cAggregate, dAggregate, eAggregate;
  PARAMETER aParameter, bParameter;
  VALUE aValue;
  intn  status;

  fseek(metadataFile, 0, 0);
  root=NewAggregate(NULL, KA_GROUP, "test", NULL);

  //Read odl, no error return 1 and error return other.
  //printf("Reading and checking odl.\n");
  status=ReadLabel(metadataFile, root);
  if (status!=1)
  {
    deleteTempFile();
    fprintf(stderr, "There are syntax errors in metadata file.\n");
    exit(1);
  }

  //Find aggregate.
  aAggregate=FindAggregate(root, attributeName);
  if (aAggregate==NULL)
  {
    deleteTempFile();
    fprintf(stderr, "No aggregate is found.\n");
    exit(1);
  }
  //PrintLabel(aAggregate);

  //Find parameter.
  aParameter=FindParameter(aAggregate, valueName);
  if (aParameter==NULL)
  {
    deleteTempFile();
    fprintf(stderr, "No parameter is found.\n");
    exit(1);
  }

  //Check value.
  aValue=FirstValue(aParameter);
  if (aValue==NULL)
  {
    deleteTempFile();
    fprintf(stderr, "No value is found.\n");
    exit(1);
  }

  if (aValue->item.type==TV_STRING)
  {
    //print string and time.
    sprintf(returnValue, "%s\n", aValue->item.value.string);
  }
  if (aValue->item.type==TV_INTEGER)
  {
    //print integer.
    sprintf(returnValue, "%i\n", aValue->item.value.integer.number);
  }

  RemoveAggregate(root);

  return (0);
}


